/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

#define NUM_OF_ITEMS 8000
#define NUM_OF_GENLIST_DEMO_NAMES 100
#define NUM_OF_GENLIST_TIMES 10

typedef struct _genlist_view_data
{
	struct appdata* ad;
	Evas_Object *btn;
	Evas_Object *layout;
	Evas_Object *ctxpopup;
	unsigned int check_cnt;
	Eina_Bool is_select_mode;
} genlist_view_data;

typedef struct _Item_Data
{
	genlist_view_data *gvd;
	int index;
	Elm_Object_Item *item;
	Evas_Object *check;
	Eina_Bool checked;
	Eina_Bool disabled;
} Item_Data;

static char *menu_its[] = {
	/*** 1line styles ***/
	"title",
	"title_with_groupindex",
	"1text",
	"1text.1icon",
	"1text.1icon.1",
	"1text.1icon.divider",
	"2text",
	"2text.1",
	"2text.1icon.1",
	"multiline/1text",
	"multiline/2text",	
	"select_mode",
	/* do not delete below */
	NULL
};

char *genlist_demo_names[] = {
	"Aaliyah", "Aamir", "Aaralyn", "Aaron", "Abagail",
	"Babitha", "Bahuratna", "Bandana", "Bulbul", "Cade", "Caldwell",
	"CaptainFantasticFasterThanSupermanSpidermanBatmanWolverineHulkAndTheFlashCombined",
	"Chandan", "Caster", "Dagan ", "Daulat", "Dag", "Earl", "Ebenzer",
	"Ellison", "Elizabeth", "Filbert", "Fitzpatrick", "Florian", "Fulton",
	"Frazer", "Gabriel", "Gage", "Galen", "Garland", "Gauhar", "Hadden",
	"Hafiz", "Hakon", "Haleem", "Hank", "Hanuman", "Jabali ", "Jaimini",
	"Jayadev", "Jake", "Jayatsena", "Jonathan", "Kamaal", "Jeirk",
	"Jasper", "Jack", "Mac", "Macy", "Marlon", "Milson",
	"Aaliyah", "Aamir", "Aaralyn", "Aaron", "Abagail",
	"Babitha", "Bahuratna", "Bandana", "Bulbul", "Cade", "Caldwell",
	"Chandan", "Caster", "Dagan ", "Daulat", "Dag", "Earl", "Ebenzer",
	"Ellison", "Elizabeth", "Filbert", "Fitzpatrick", "Florian", "Fulton",
	"Frazer", "Gabriel", "Gage", "Galen", "Garland", "Gauhar", "Hadden",
	"Hafiz", "Hakon", "Haleem", "Hank", "Hanuman", "Jabali ", "Jaimini",
	"Jayadev", "Jake", "Jayatsena", "Jonathan", "Kamaal", "Jeirk",
	"Jasper", "Jack", "Mac", "Macy", "Marlon", "Milson",
	NULL
};

char *genlist_times[] = {"10:00 AM", "10:01 AM", "10:02 AM", "10:03 AM", "10:04 AM", "10:05 AM",
	"3:50 PM", "3:51 PM", "3:52 PM", "3:53 PM"};

char *genlist_messages[] = {"Good morning!",
	"It's you again. How do you do this?",
	"This is the princess who fell from the sky into my arms.",
	"Yes. We keep meeting like this. You just suddenly show up.",
	"We could make plans to meet.",
	"No, it's nicer this way. I hope we meet again suddenly.",
	"Don't you know that it takes so little to make me happy?",
	"Good morning, Princess!",
	"This is incredible. You owe me an explanation.",
	"No, you're the one who owes me an explanation. You've really got a crush on me. Where shall we go, Princess?"
};

static void _gl_del(void *data, Evas_Object *obj)
{
	// FIXME: Unrealized callback can be called after this.
	// Accessing Item_Data can be dangerous on unrealized callback.
	Item_Data *id = (Item_Data *)data;
	if (id) free(id);
}

static void
_gl_ctxpopup_del_cb(void *data,  Evas *e, Evas_Object *obj, void *event_info)
{
	genlist_view_data *gvd = (genlist_view_data *) data;
	if (gvd) free(gvd);
}

static Evas_Object *
_gl_icon_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *icon = NULL;
	Item_Data *id = (Item_Data *)data;

	if (!strcmp(part, "elm.swallow.colorbar")) {
		if (id->index == 1) {
			icon = evas_object_rectangle_add(evas_object_evas_get(obj));
			evas_object_color_set(icon, 255, 0, 0, 255);
			evas_object_size_hint_min_set(icon, 7 * elm_config_scale_get(), 0);
			evas_object_size_hint_max_set(icon, 7 * elm_config_scale_get(), -1);
		}
		return icon;
	} else if (!strcmp(part, "elm.icon.noti")) {
		if ((id->index % 4) == 0 || ((id->index % 4) == 1)) {
			icon = elm_image_add(obj);
			elm_image_file_set(icon, ICON_DIR"/b_listnoti_popupbg.png", NULL);
			elm_image_aspect_fixed_set(icon, EINA_FALSE);
		if ((id->index % 4) == 0) {
				evas_object_size_hint_min_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			} else if ((id->index % 4) == 1) {
				evas_object_size_hint_min_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			}
		} else return NULL;
		return icon;
	} else if (!strcmp(part, "elm.swallow.center_check")) {
		return NULL;
	}

	icon = elm_image_add(obj);
	elm_image_file_set(icon, ICON_DIR"/g1.png", NULL);
	evas_object_size_hint_min_set(icon, 56, 56);
	evas_object_size_hint_max_set(icon, 56, 56);

	return icon;
}

static void
_gl_check_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Item_Data *id = (Item_Data *)data;
	char buf[5];
	id->gvd->check_cnt += (id->checked)? 1:-1;
	snprintf(buf, 5, "%.2d", id->gvd->check_cnt);
	elm_object_text_set(id->gvd->btn, buf);
}

static Evas_Object *
_gl_noti_icon_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *icon = NULL;
	Item_Data *id = (Item_Data *)data;
	if (!strcmp(part, "elm.icon.noti")) {
		if ((id->index % 4) == 0 || ((id->index % 4) == 1)) {
			icon = elm_image_add(obj);
			elm_image_file_set(icon, ICON_DIR"/b_listnoti_popupbg.png", NULL);
			elm_image_aspect_fixed_set(icon, EINA_FALSE);
		if ((id->index % 4) == 0) {
				evas_object_size_hint_min_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			} else if ((id->index % 4) == 1) {
				evas_object_size_hint_min_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			}
		} else return NULL;
		return icon;
	} else
		return _gl_icon_get(data, obj, part);
}

static Evas_Object *
_gl_divider_icon_get(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *btn, *ic;
	Evas_Object *icon = NULL;
	Item_Data *id = (Item_Data *)data;
	if (!strcmp(part, "elm.swallow.colorbar")) {
		if (id->index == 1) {
			icon = evas_object_rectangle_add(evas_object_evas_get(obj));
			evas_object_color_set(icon, 255, 0, 0, 255);
			evas_object_size_hint_min_set(icon, 7 * elm_config_scale_get(), 0);
			evas_object_size_hint_max_set(icon, 7 * elm_config_scale_get(), -1);
		}
		return icon;
	} else if (!strcmp(part, "elm.icon.noti")) {
		if ((id->index % 4) == 0 || ((id->index % 4) == 1)) {
			icon = elm_image_add(obj);
			elm_image_file_set(icon, ICON_DIR"/b_listnoti_popupbg.png", NULL);
			elm_image_aspect_fixed_set(icon, EINA_FALSE);
			if ((id->index % 4) == 0) {
				evas_object_size_hint_min_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 45 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			} else if ((id->index % 4) == 1) {
				evas_object_size_hint_min_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
					evas_object_size_hint_max_set(icon, 30 * elm_config_scale_get(),
					30 * elm_config_scale_get());
			}
		} else return NULL;
		return icon;
	} else if (!strcmp(part, "elm.swallow.center_check")) {
		return NULL;
	}

	btn = elm_button_add(obj);
	elm_object_style_set(btn, "list_divider_btn");
	evas_object_size_hint_min_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);

	ic = elm_image_add(btn);
	elm_image_file_set(ic, ICON_DIR "/tw_btn_delete_holo_dark.png", NULL);
	evas_object_size_hint_min_set(ic, 56, 56);
	evas_object_size_hint_max_set(ic, 56, 56);
	elm_object_content_set(btn, ic);
	evas_object_propagate_events_set(btn, EINA_FALSE);

	return btn;
}

static char *
_gl_text_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[1024];
	Item_Data *id = (Item_Data *)data;
	int index = id->index;

	if (!strcmp(part, "elm.text.1") || !strcmp(part, "elm.text.2")) index++;

	snprintf(buf, 1023, "%s:%s", part, genlist_demo_names[index%NUM_OF_GENLIST_DEMO_NAMES]);
	return strdup(buf);
}

static char* _gl_multiline_1text_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = (Item_Data *)data;
	int index = id->index % NUM_OF_GENLIST_TIMES;

	if (!strcmp(part, "elm.text") ) {
		return strdup(genlist_messages[index]);
	}
	return NULL;
}

static char* _gl_multiline_2text_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = (Item_Data *)data;
	int index = id->index % NUM_OF_GENLIST_TIMES;

	if (!strcmp(part, "elm.text") ) {
		return strdup(genlist_times[index]);
	} else {
		return strdup(genlist_messages[index]);
	}
	return NULL;
}

static char *
_gl_noti_text_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = (Item_Data *)data;
	if (!strcmp(part, "elm.text.noti")) {
		if ((id->index % 4) == 0) return strdup("999");
		else if ((id->index % 4) == 1) return strdup("N");
		else return NULL;
	}
	return _gl_text_get(data, obj, part);
}

static void
_gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	elm_genlist_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);
	return;
}

static void
_gl_select_mode_sel(void *data, Evas_Object *obj, void *event_info)
{
	genlist_view_data *gvd = (genlist_view_data *) data;
	elm_genlist_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);

	if (!gvd->is_select_mode) return;

	Item_Data *id = (Item_Data *)elm_object_item_data_get((Elm_Object_Item *)event_info);
	char buf[5];
	id->checked = !id->checked;
	elm_check_state_set(id->check, id->checked);
	gvd->check_cnt += (id->checked)? 1:-1;
	snprintf(buf, 5, "%.2d", gvd->check_cnt);
	elm_object_text_set(gvd->btn, buf);
}

static void
_gl_longpress_cb(void *data, Evas_Object *obj, void *event_info)
{
	genlist_view_data *gvd = (genlist_view_data *) data;
	elm_genlist_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);

	if (!gvd->is_select_mode) {
		gvd->is_select_mode = EINA_TRUE;
		Item_Data *id = (Item_Data *)elm_object_item_data_get((Elm_Object_Item *)event_info);
		id->checked = !id->checked;
		elm_genlist_realized_items_update(obj);

		char buf[5];
		gvd->check_cnt += (id->checked)? 1:-1;
		snprintf(buf, 5, "%.2d", gvd->check_cnt);
		elm_object_text_set(gvd->btn, buf);
		elm_object_signal_emit(gvd->layout, "select_mode,button,show", "");
	}
}

static Evas_Object *
_gl_check_get(void *data, Evas_Object *obj, const char *part)
{
	Item_Data *id = (Item_Data *)data;

	if ((!id->gvd->is_select_mode) || strcmp(part, "elm.swallow.center_check")) return NULL;

	id->check = elm_check_add(obj);
	elm_object_style_set(id->check, "genlist/select_mode");
	elm_check_state_set(id->check, EINA_FALSE);
	evas_object_smart_callback_add(id->check, "changed", _gl_check_clicked_cb, data);
	elm_check_state_pointer_set(id->check, &id->checked);
	evas_object_repeat_events_set(id->check, EINA_FALSE);
	evas_object_propagate_events_set(id->check, EINA_FALSE);
	evas_object_show(id->check);

	return id->check;
}

static void
_gl_ctxpopup_select_all_cb(void *data, Evas_Object *obj, void *event_info)
{
	char buf[5];
	genlist_view_data *gvd = (genlist_view_data *)data;
	Evas_Object *layout = gvd->layout;
	Evas_Object *genlist = elm_layout_content_get(layout, "elm.swallow.content");
	Evas_Object *select_btn = gvd->btn;
	Evas_Object *ctxpopup = gvd->ctxpopup;
	Elm_Object_Item *it = elm_genlist_first_item_get(genlist);

	while(it) {
		Item_Data *id = (Item_Data *)elm_object_item_data_get(it);
		// For realized items, set state of real check object
		Evas_Object *ck = elm_object_item_part_content_get(it, "elm.swallow.center_check");
		if (ck) elm_check_state_set(ck, EINA_TRUE);
		// For all items (include unrealized), just set pointer state
		id->checked = EINA_TRUE;
		it = elm_genlist_item_next_get(it);
	}
	gvd->check_cnt = elm_genlist_items_count(genlist);
	snprintf(buf, 5, "%.2d", gvd->check_cnt);
	elm_object_text_set(select_btn, buf);

	if (ctxpopup)
		elm_ctxpopup_dismiss(ctxpopup);
}

static void
_gl_ctxpopup_deselect_all_cb(void *data, Evas_Object *obj, void *event_info)
{
	char buf[5];
	genlist_view_data *gvd = (genlist_view_data *)data;
	Evas_Object *layout = gvd->layout;
	Evas_Object *genlist = elm_layout_content_get(layout, "elm.swallow.content");
	Evas_Object *select_btn = gvd->btn;
	Evas_Object *ctxpopup = gvd->ctxpopup;

	Elm_Object_Item *it = elm_genlist_first_item_get(genlist);
	while(it) {
		Item_Data *id = (Item_Data *)elm_object_item_data_get(it);
		// For realized items, set state of real check object
		Evas_Object *ck = elm_object_item_part_content_get(it, "elm.swallow.center_check");
		if (ck) elm_check_state_set(ck, EINA_FALSE);
		// For all items (include unrealized), just set pointer state
		id->checked = EINA_FALSE;
		it = elm_genlist_item_next_get(it);
	}
	gvd->check_cnt = 0;
	snprintf(buf, 5, "%.2d", gvd->check_cnt);
	elm_object_text_set(select_btn, buf);


	if (ctxpopup)
		elm_ctxpopup_dismiss(ctxpopup);
}

static void
_gl_dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *ctxpopup = (Evas_Object *)data;

	evas_object_smart_callback_del(ctxpopup, "dismissed", _gl_dismissed_cb);
	evas_object_del(ctxpopup);
}

static void
_gl_move_ctxpopup(Evas_Object *ctxpopup, Evas_Object *btn)
{
	Evas_Coord x, y, w, h;
	evas_object_geometry_get(btn, &x, &y, &w, &h);
	evas_object_move(ctxpopup, x + w/2 - 22, y + h - 8);
}

static void
_create_ctxpopup_action_button(void *data, Evas_Object *obj, void *event_info)
{
	genlist_view_data *gvd = (genlist_view_data *) data;
	Evas_Object *layout = gvd->layout;
	Evas_Object *ctxpopup = NULL;
	Elm_Object_Item *eo_it1 = NULL;
	Elm_Object_Item *eo_it2 = NULL;

	ctxpopup = elm_ctxpopup_add(layout);
	/* elm_object_style_set(ctxpopup, "edit.button"); */
	elm_object_style_set(ctxpopup, "default");
	elm_ctxpopup_direction_priority_set(ctxpopup, ELM_CTXPOPUP_DIRECTION_DOWN,
						ELM_CTXPOPUP_DIRECTION_DOWN,
						ELM_CTXPOPUP_DIRECTION_DOWN,
						ELM_CTXPOPUP_DIRECTION_DOWN);
	eext_object_event_callback_add(ctxpopup, EEXT_CALLBACK_BACK, eext_ctxpopup_back_cb, NULL);
	evas_object_smart_callback_add(ctxpopup, "dismissed", _gl_dismissed_cb, ctxpopup);

	elm_layout_content_get(layout, "elm.swallow.content");
	gvd->layout = layout;
	gvd->ctxpopup = ctxpopup;
	eo_it1 = elm_ctxpopup_item_append(ctxpopup, _("Select all"), NULL, _gl_ctxpopup_select_all_cb, gvd);
	eo_it2 = elm_ctxpopup_item_append(ctxpopup, _("Deselect all"), NULL, _gl_ctxpopup_deselect_all_cb, gvd);

	if (eo_it1 && eo_it2) {
		elm_object_item_style_set(eo_it1, "select_mode");
		elm_object_item_style_set(eo_it2, "select_mode");
	}

	evas_object_size_hint_max_set(ctxpopup, 320, 189);
	_gl_move_ctxpopup(ctxpopup, obj);
	evas_object_show(ctxpopup);
}

static Evas_Object *
_create_select_mode_genlist(genlist_view_data *gvd)
{
	char buf[5];
	Evas_Object * genlist = NULL;

	gvd->layout = elm_layout_add(gvd->ad->nf);
	elm_layout_theme_set(gvd->layout, "layout", "select_mode", "default");
	evas_object_show(gvd->layout);
	genlist = elm_genlist_add(gvd->layout);
	elm_layout_content_set(gvd->layout, "elm.swallow.content", genlist);

	evas_object_smart_callback_add(genlist, "longpressed", _gl_longpress_cb, gvd);
	elm_genlist_longpress_timeout_set(genlist, 1.0);

	gvd->btn = elm_button_add(gvd->layout);
	elm_object_style_set(gvd->btn, "select_mode");
	snprintf(buf, 5, "%.2d", gvd->check_cnt);
	elm_object_part_text_set(gvd->btn, "elm.text", buf);
	evas_object_smart_callback_add(gvd->btn, "clicked", _create_ctxpopup_action_button, gvd);

	elm_layout_content_set(gvd->layout, "elm.swallow.icon", gvd->btn);

	return genlist;
}

static void _create_genlist(void *data, Evas_Object *obj, void *event_info)
{
	if (!data) return;
	struct appdata *ad = (struct appdata *)data;
	genlist_view_data *gvd;
	int index;
	Elm_Object_Item *item;
	Evas_Object *genlist;
	const char *style = elm_object_item_text_get(event_info);

	gvd = (genlist_view_data *)calloc(1, sizeof(genlist_view_data));
	gvd->ad = ad;

	Elm_Genlist_Item_Class *gtc = elm_genlist_item_class_new();
	Elm_Genlist_Item_Class *ttc = elm_genlist_item_class_new();

	// Create item class
	Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();

	if (!strcmp(style, "title") || !strcmp(style, "title_with_groupindex"))
		itc->item_style = "1text";
	else if (!strcmp(style, "select_mode"))
		itc->item_style = "1text";
	else
		itc->item_style = style;

	itc->func.text_get = _gl_text_get;
	itc->func.content_get = _gl_icon_get;
	itc->func.del = _gl_del;

	if (!strcmp(style, "title"))
	{
		gtc->item_style = "title";
		gtc->func.text_get = _gl_text_get;
		gtc->func.content_get = _gl_icon_get;
		gtc->func.del = _gl_del;
	}
	else if (!strcmp(style, "title_with_groupindex"))
	{
		gtc->item_style = "title";
		gtc->func.text_get = _gl_text_get;
		gtc->func.content_get = _gl_icon_get;
		gtc->func.del = _gl_del;

		ttc->item_style = "groupindex";
		ttc->func.text_get = _gl_text_get;
		ttc->func.content_get = _gl_icon_get;
		ttc->func.del = _gl_del;
	}

	if(!strcmp(style, "select_mode"))
	{
		gvd->layout = elm_layout_add(ad->nf);
		genlist = _create_select_mode_genlist(gvd);
		itc->func.content_get = _gl_check_get;
		elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
		evas_object_event_callback_add(genlist, EVAS_CALLBACK_DEL, _gl_ctxpopup_del_cb, (void *) gvd);
	}
	else
		genlist = elm_genlist_add(ad->nf);

	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);

	// HOMOGENEOUS MODE
	// If item height is same when each style name is same,
	// Use homogeneous mode.
	if (!strstr(style, "multiline")) {
		elm_genlist_homogeneous_set(genlist, EINA_TRUE);
	}

	if (!strcmp(style, "multiline/1text")) {
		itc->func.text_get = _gl_multiline_1text_get;
	} else if (!strcmp(style, "multiline/2text")) {
		itc->func.text_get = _gl_multiline_2text_get;
	} else if (!strcmp(style, "2text") || !strcmp(style, "3text")) {
		itc->func.text_get = _gl_noti_text_get;
		itc->func.content_get = _gl_noti_icon_get;
	} else if (strstr(style, "divider")) {
		itc->func.content_get = _gl_divider_icon_get;
	}

	for (index = 0; index < NUM_OF_ITEMS; index++) {
		Item_Data *id = calloc(sizeof(Item_Data), 1);
		id->index = index;
		id->gvd = gvd;

		if (!strcmp(style, "title"))
		{
			if (index == 0)
				item = elm_genlist_item_append(
					genlist,			// genlist object
					gtc,				// item class
					id,					// data
					NULL,
					ELM_GENLIST_ITEM_GROUP,
					_gl_sel,
					NULL);
			else
				item = elm_genlist_item_append(
					genlist,			// genlist object
					itc,				// item class
					id,					// data
					NULL,
					ELM_GENLIST_ITEM_NONE,
					_gl_sel,
					NULL);
		}
		else if(!strcmp(style, "title_with_groupindex"))
		{
			if (index == 0)
				item = elm_genlist_item_append(
					genlist,			// genlist object
					gtc,				// item class
					id,					// data
					NULL,
					ELM_GENLIST_ITEM_GROUP,
					_gl_sel,
					NULL);
			else if (index == 1)
				item = elm_genlist_item_append(
					genlist,			// genlist object
					ttc,				// item class
					id,					// data
					NULL,
					ELM_GENLIST_ITEM_GROUP,
					_gl_sel,
					NULL);
			else
				item = elm_genlist_item_append(
					genlist,			// genlist object
					itc,				// item class
					id,					// data
					NULL,
					ELM_GENLIST_ITEM_NONE,
					_gl_sel,
					NULL);
		}
		else if (!strcmp(style, "select_mode"))
			item = elm_genlist_item_append(
				genlist,			// genlist object
				itc,				// item class
				id,					// data
				NULL,
				ELM_GENLIST_ITEM_NONE,
				_gl_select_mode_sel,
				gvd);
		else
			item = elm_genlist_item_append(
				genlist,			// genlist object
				itc,				// item class
				id,					// data
				NULL,
				ELM_GENLIST_ITEM_NONE,
				_gl_sel,
				NULL);

		id->item = item;

		if (!strcmp(itc->item_style, "groupindex"))
			elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}

	elm_genlist_item_class_free(itc);

	if (!strcmp(style, "select_mode"))
		elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, gvd->layout, "empty");
	else
		elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, genlist, "empty");
}

static void _list_click(void *data, Evas_Object * obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) elm_list_selected_item_get(obj);

	if (!it) {
		fprintf(stderr, "list item is NULL\n");
		return;
	}

	elm_list_item_selected_set(it, EINA_FALSE);
}

void genlist_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *list;
	int i;

	ad = (struct appdata *) data;
	if (ad == NULL) return;

	list = elm_list_add(ad->nf);
	elm_list_mode_set(list, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(list, "selected", _list_click, NULL);

	for (i = 0; menu_its[i]; i++) {
		elm_list_item_append(list, menu_its[i], NULL, NULL,
				_create_genlist, ad);
	}
	elm_list_go(list);

	elm_naviframe_item_push(ad->nf, "Genlist", NULL, NULL, list, "empty");
}
