/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

int center_item = 0;

#define NUM_ITEMS 7
#define NUM_INDEX 7
typedef struct _page_data page_data;
struct _page_data
{
	Evas_Object *scroller;
	Evas_Object *mapbuf[NUM_ITEMS];
	Evas_Object *index;
	int cur_page;
	Elm_Object_Item *it[NUM_ITEMS];

	Elm_Object_Item *last_it;
	Elm_Object_Item *new_it;
	int min_page, max_page;
};

static void _index_sync(void *data);

static void
_on_index_mouse_down_cb(void *data, Evas *e, Evas_Object *o, void *event_info)
{
	page_data *pd = (page_data *)data;

	/* Keep the last index item active and save the selected index item */
	if (!pd->last_it) return;

	int level = elm_index_item_level_get(o);
	pd->new_it = elm_index_selected_item_get(o, level);
	elm_index_item_selected_set(pd->last_it, EINA_TRUE);
}

static void
_on_index_mouse_up_cb(void *data, Evas *e, Evas_Object *o, void *event_info)
{
	page_data *pd = (page_data *)data;

	/* Keep the last index item active and move to the page of the currently selected index item */
	if (!pd->last_it) return;
	elm_index_item_selected_set(pd->last_it, EINA_TRUE);

	if (!pd->new_it) return;

	int idx = (int) elm_object_item_data_get(pd->new_it);
	if (idx == pd->cur_page) return;

	elm_scroller_page_bring_in(pd->scroller, idx, 0);
}

static void
_on_index_mouse_move_cb(void *data, Evas *e, Evas_Object *o, void *event_info)
{
	page_data *pd = (page_data *)data;

	/* Keep the last index item active and save the currently selected index item */
	if (!pd->last_it) return;

	int level = elm_index_item_level_get(o);
	pd->new_it = elm_index_selected_item_get(o, level);
	elm_index_item_selected_set(pd->last_it, EINA_TRUE);
}

static void
_index_refresh(void *data)
{
	int i;
	page_data *pd = (page_data *)data;

	elm_index_item_clear(pd->index);
	if (pd->cur_page < pd->min_page)
	{
		for(i = pd->cur_page ; i < pd->cur_page + NUM_INDEX ; i++)
		{
			printf("Added : %d\n", i);
			elm_index_item_append(pd->index, NULL, NULL, (void *) i);
		}
		pd->min_page = pd->cur_page;
		pd->min_page = pd->cur_page + NUM_INDEX - 1;
	}
	else
	{
		for(i = pd->cur_page - NUM_INDEX + 1; i < pd->cur_page + 1 ; i++)
		{
			printf("Added : %d\n", i);
			elm_index_item_append(pd->index, NULL, NULL, (void *) i);
		}
		pd->min_page = pd->cur_page - NUM_INDEX;
		pd->min_page = pd->cur_page - 1;
	}
	elm_index_level_go(pd->index, 0);
	_index_sync(pd);
}

static void
_index_sync(void *data)
{
	page_data *pd = (page_data *)data;
	Elm_Object_Item *it;
	it = elm_index_item_find(pd->index, (void *)pd->cur_page);
	if (it) {
		elm_index_item_selected_set(it, EINA_TRUE);
		pd->last_it = it;
		pd->new_it = it;
	}
	else
		_index_refresh(pd);
}

static void
_layout_del_cb(void *data , Evas *e, Evas_Object *obj, void *event_info)
{
	page_data *pd = data;
	free(pd);
}

static Eina_Bool
animator_cb(void *data)
{
	int i;
	page_data *pd = (page_data *)data;

	//Since mapbuf is not perfect, Enable them after the size calculation is finished
	for (i = 0; i < NUM_ITEMS; ++i)
	{
		elm_mapbuf_enabled_set(pd->mapbuf[i], EINA_TRUE);
	}
	return ECORE_CALLBACK_CANCEL;
}

static void
_scroll(void *data, Evas_Object *obj, void *ei)
{
	int cur_page;
	page_data *pd = data;
	elm_scroller_current_page_get(pd->scroller, &cur_page, NULL);
	if (cur_page != pd->cur_page) {
		printf("scroll: %d\n", pd->cur_page);
		pd->cur_page = cur_page;
		_index_sync(pd);
	}
}

static void
_body_thumbnail_queue_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *layout, *scroller, *box, *page_layout, *page, *rect;
	int i;

	if (!data) return;
	ad = data;

	page_data *pd = calloc(1, sizeof(page_data));
	/* Create Layout */
	layout = elm_layout_add(ad->nf);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/index_thumbnail");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(layout);
	evas_object_event_callback_add(layout, EVAS_CALLBACK_DEL, _layout_del_cb, pd);

	/* Create Scroller */
	scroller = elm_scroller_add(layout);
	elm_scroller_loop_set(scroller, EINA_FALSE, EINA_FALSE);
	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_scroller_page_size_set(scroller, 248, 0);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
	elm_scroller_page_scroll_limit_set(scroller, 1, 0);
	elm_object_scroll_lock_y_set(scroller, EINA_TRUE);
	elm_object_part_content_set(layout, "scroller", scroller);
	evas_object_show(scroller);
	pd->scroller = scroller;

	/* Create Box */
	box = elm_box_add(scroller);
	elm_box_horizontal_set(box, EINA_TRUE);
	elm_object_content_set(scroller, box);
	evas_object_show(box);

	rect = evas_object_rectangle_add(evas_object_evas_get(box));
	evas_object_size_hint_min_set(rect, 56, 480);
	elm_box_pack_end(box, rect);

	/* Create Pages */
	for (i = 0; i < NUM_ITEMS; ++i)
	{
		page_layout = elm_layout_add(box);
		evas_object_size_hint_weight_set(page_layout, 0, 0);
		evas_object_size_hint_align_set(page_layout, 0, EVAS_HINT_FILL);
		elm_layout_theme_set(page_layout, "layout", "body_thumbnail", "default");
		evas_object_show(page_layout);

		page = evas_object_rectangle_add(evas_object_evas_get(page_layout));
		if (i % 3 == 0)
			evas_object_color_set(page, 0, 0, 50, 50);
		else if (i % 3 == 1)
		{
			evas_object_color_set(page, 0, 50, 0, 50);
			elm_object_part_text_set(page_layout, "elm.text", "Notification");
		}
		else
		{
			evas_object_color_set(page, 50, 0, 0, 50);
			elm_object_part_text_set(page_layout, "elm.text", "Notification Notification");
		}
		evas_object_size_hint_weight_set(page, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(page, EVAS_HINT_FILL, EVAS_HINT_FILL);

		elm_object_part_content_set(page_layout, "elm.icon", page);
		if (i)
			elm_object_signal_emit(page_layout, "elm,state,thumbnail,unfocus", "elm");
		elm_box_pack_end(box, page_layout);
	}

	elm_box_pack_end(box, rect);
	ecore_animator_add(animator_cb, pd);

	/* Use Index */
	evas_object_smart_callback_add(scroller, "scroll", _scroll, pd);
	Evas_Object *index;
	index = elm_index_add(layout);
	elm_object_style_set(index, "thumbnail");
	evas_object_size_hint_weight_set(index, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(index, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_index_horizontal_set(index, EINA_TRUE);
	elm_index_autohide_disabled_set(index, EINA_TRUE);
	elm_object_part_content_set(layout, "controller", index);

	center_item = 6;

	for (i = 0; i < NUM_INDEX; ++i)
	{
      pd->it[i] = elm_index_item_append(index, NULL, NULL, (void *) i);
	}

	pd->min_page = 0;
	pd->max_page = NUM_INDEX - 1;

	elm_index_level_go(index, 0);
	elm_index_item_selected_set(pd->it[0], EINA_TRUE);
	pd->cur_page = 0;
	pd->index = index;
	pd->last_it = pd->it[0];

	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_DOWN, _on_index_mouse_down_cb, pd);
	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_MOVE, _on_index_mouse_move_cb, pd);
	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_UP, _on_index_mouse_up_cb, pd);

	Elm_Object_Item *it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, layout, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void
_tab_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *layout, *scroller, *box, *page_layout, *page, *mapbuf, *rect;

	if (!data) return;
	ad = data;

	page_data *pd = calloc(1, sizeof(page_data));
	/* Create Layout */
	layout = elm_layout_add(ad->nf);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/index_tab");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(layout);
	evas_object_event_callback_add(layout, EVAS_CALLBACK_DEL, _layout_del_cb, pd);

	/* Create Scroller */
	scroller = elm_scroller_add(layout);
	elm_scroller_loop_set(scroller, EINA_FALSE, EINA_FALSE);
	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_scroller_page_size_set(scroller, 248, 0);
	elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
	elm_scroller_page_scroll_limit_set(scroller, 1, 0);
	elm_object_scroll_lock_y_set(scroller, EINA_TRUE);
	elm_object_part_content_set(layout, "scroller", scroller);
	evas_object_show(scroller);
	pd->scroller = scroller;

	/* Create Box */
	box = elm_box_add(scroller);
	elm_box_horizontal_set(box, EINA_TRUE);
	elm_object_content_set(scroller, box);
	evas_object_show(box);

	rect = evas_object_rectangle_add(evas_object_evas_get(box));
	evas_object_size_hint_min_set(rect, 56, 480);
	elm_box_pack_end(box, rect);

	/* Create Pages */
	// page 1 layout
	page_layout = elm_layout_add(box);
	elm_layout_theme_set(page_layout, "layout", "body_thumbnail", "default");
	evas_object_size_hint_weight_set(page_layout, 0, 0);
	evas_object_size_hint_align_set(page_layout, 0, EVAS_HINT_FILL);
	evas_object_show(page_layout);

	// page 1 content
	page = evas_object_rectangle_add(evas_object_evas_get(page_layout));
	evas_object_color_set(page, 50, 0, 0, 50);
	evas_object_size_hint_weight_set(page, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(page, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(page_layout, "elm.icon", page);

	// mapbuf for page 1
	mapbuf = elm_mapbuf_add(box);
	evas_object_size_hint_weight_set(mapbuf, 0, 0);
	evas_object_size_hint_align_set(mapbuf, 0, EVAS_HINT_FILL);
	evas_object_show(mapbuf);
	elm_object_content_set(mapbuf, page_layout);
	pd->mapbuf[0] = mapbuf;
	elm_box_pack_end(box, mapbuf);

	// page 2 layout
	page_layout = elm_layout_add(box);
	evas_object_size_hint_weight_set(page_layout, 0, 0);
	evas_object_size_hint_align_set(page_layout, 0, EVAS_HINT_FILL);
	elm_layout_theme_set(page_layout, "layout", "body_thumbnail", "default");
	evas_object_show(page_layout);

	// page 2 content
	page = evas_object_rectangle_add(evas_object_evas_get(page_layout));
	evas_object_color_set(page, 0, 50, 0, 50);
	evas_object_size_hint_weight_set(page, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(page, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(page_layout, "elm.icon", page);

	// mapbuf for page 2
	mapbuf = elm_mapbuf_add(box);
	evas_object_size_hint_weight_set(mapbuf, 0, 0);
	evas_object_size_hint_align_set(mapbuf, 0, EVAS_HINT_FILL);
	evas_object_show(mapbuf);
	elm_object_content_set(mapbuf, page_layout);
	pd->mapbuf[1] = mapbuf;
	elm_box_pack_end(box, mapbuf);

	// page 3 layout
	page_layout = elm_layout_add(box);
	evas_object_size_hint_weight_set(page_layout, 0, 0);
	evas_object_size_hint_align_set(page_layout, 0, EVAS_HINT_FILL);
	elm_layout_theme_set(page_layout, "layout", "body_thumbnail", "default");
	evas_object_show(page_layout);

	// page 3 content
	page = evas_object_rectangle_add(evas_object_evas_get(page_layout));
	evas_object_color_set(page, 0, 0, 50, 50);
	evas_object_size_hint_weight_set(page, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(page, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_part_content_set(page_layout, "elm.icon", page);

	// mapbuf for page 3
	mapbuf = elm_mapbuf_add(box);
	evas_object_size_hint_weight_set(mapbuf, 0, 0);
	evas_object_size_hint_align_set(mapbuf, 0, EVAS_HINT_FILL);
	evas_object_show(mapbuf);
	elm_object_content_set(mapbuf, page_layout);
	pd->mapbuf[2] = mapbuf;
	elm_box_pack_end(box, mapbuf);

	elm_box_pack_end(box, rect);

	ecore_animator_add(animator_cb, pd);

	/* Use Index */
	evas_object_smart_callback_add(scroller, "scroll", _scroll, pd);
	Evas_Object *index;
	index = elm_index_add(layout);
	elm_object_style_set(index, "tab");
	evas_object_size_hint_weight_set(index, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(index, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_index_horizontal_set(index, EINA_TRUE);
	elm_index_autohide_disabled_set(index, EINA_TRUE);
	elm_object_part_content_set(layout, "controller", index);

	pd->it[0] = elm_index_item_append(index, "1", NULL, (void *) 0);
	pd->it[1] = elm_index_item_append(index, "2", NULL, (void *) 1);
	pd->it[2] = elm_index_item_append(index, "3", NULL, (void *) 2);

	elm_index_level_go(index, 0);
	elm_index_item_selected_set(pd->it[0], EINA_TRUE);
	pd->cur_page = 0;
	pd->index = index;
	pd->last_it = pd->it[0];

	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_DOWN, _on_index_mouse_down_cb, pd);
	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_MOVE, _on_index_mouse_move_cb, pd);
	evas_object_event_callback_add(index, EVAS_CALLBACK_MOUSE_UP, _on_index_mouse_up_cb, pd);

	Elm_Object_Item *it = elm_naviframe_item_push(ad->nf, NULL, NULL, NULL, layout, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void _list_click(void *data, Evas_Object * obj, void *ei)
{
	elm_list_item_selected_set(ei, EINA_FALSE);
}

void index_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad = data;
	Evas_Object *list;

	list = elm_list_add(ad->nf);
	elm_list_mode_set(list, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(list, "selected", _list_click, NULL);
	elm_list_item_append(list, "Horizontal Style", NULL, NULL, _body_thumbnail_queue_cb, ad);
	elm_list_item_append(list, "Tab Style", NULL, NULL, _tab_cb, ad);
	elm_list_go(list);

	elm_naviframe_item_push(ad->nf, "PageControl", NULL, NULL, list, NULL);
}
