/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
 label
 ********************************************************/

static void
_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *label = (Evas_Object *)data;
	const char* text;
	text = elm_object_text_get(obj);
	if (strcmp(text,"Start Sliding") == 0)
	{
		while (label != NULL)
			{
				label = evas_object_data_get(label, "next_label");
				elm_label_slide_mode_set(label, ELM_LABEL_SLIDE_MODE_AUTO);
				elm_label_slide_go(label);
			}
		elm_object_text_set(obj, "Stop Sliding");
	}
	else
	{
		while (label != NULL)
			{
				label = evas_object_data_get(label, "next_label");
				elm_label_slide_mode_set(label, ELM_LABEL_SLIDE_MODE_NONE);
				elm_label_slide_go(label);
			}
		elm_object_text_set(obj, "Start Sliding");
	}
}

static Evas_Object* _create_labels(Evas_Object* parent)
{
	Evas_Object* layout = elm_layout_add(parent);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "elmdemo-test/label_slide");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	Evas_Object *label_rollShort;
	label_rollShort = elm_label_add(layout);
	elm_object_style_set(label_rollShort, "slide_roll");
	elm_object_text_set(label_rollShort, "<color=#4D554BFF>Roll style.</color>");
	elm_object_part_content_set(layout, "swallow_label_5", label_rollShort);
	elm_label_wrap_width_set(label_rollShort, 100);
	elm_label_slide_mode_set(label_rollShort, ELM_LABEL_SLIDE_MODE_ALWAYS);

	Evas_Object *label_rollLong;
	label_rollLong = elm_label_add(layout);
	elm_object_style_set(label_rollLong, "slide_roll");
	elm_object_text_set(label_rollLong, "<color=#4D554BFF>This is a sample long text for slide roll style demo in elm label.</color>");
	elm_object_part_content_set(layout, "swallow_label_6", label_rollLong);
	elm_label_wrap_width_set(label_rollLong, 100);
	elm_label_slide_mode_set(label_rollLong, ELM_LABEL_SLIDE_MODE_ALWAYS);

	evas_object_data_set(label_rollShort, "next_label", label_rollLong);

	Evas_Object *btn;
	btn = elm_button_add(layout);
	elm_object_text_set(btn, "Start Sliding");
	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_smart_callback_add(btn, "clicked", _btn_clicked_cb, label_rollShort);
	elm_object_part_content_set(layout, "swallow_btn", btn);

	return layout;
}

static Evas_Object* _create_scroller(Evas_Object* parent)
{
	Evas_Object* scroller = elm_scroller_add(parent);
	elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
	elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_OFF,ELM_SCROLLER_POLICY_AUTO);
	evas_object_show(scroller);

	return scroller;
}

void label_slide_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *scroller, *layout_inner;
	struct appdata *ad = (struct appdata *) data;
	if (ad == NULL) return;

	scroller = _create_scroller(ad->nf);
	elm_naviframe_item_push(ad->nf, _("Slide Text"), NULL, NULL, scroller, NULL);

	layout_inner = _create_labels(ad->nf);
	elm_object_content_set(scroller, layout_inner);
}
