/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

/*********************************************************
 SearchBar Normal
 ********************************************************/
static void _nocontent_cb(void *data, Evas_Object *obj, void *event_info);
static void _nocontent_title_enabled_cb(void *data, Evas_Object *obj, void *event_info);

static struct _menu_item menu_its[] = {
	{ "Nocontent", _nocontent_cb },
	{ "Title Enabled", _nocontent_title_enabled_cb },
	{ NULL, NULL }
};

static void _nocontent_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *ly, *rect;
	Elm_Object_Item *it;
	struct appdata *ad = (struct appdata *)data;
	if (ad == NULL) return;

	ly = elm_layout_add(obj);
	elm_layout_theme_set(ly, "layout", "nocontents", "default");
	evas_object_size_hint_weight_set (ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);
	rect = evas_object_rectangle_add(evas_object_evas_get(obj));
	evas_object_show(rect);
	elm_object_part_content_set(ly, "elm.swallow.icon", rect);
	elm_object_part_text_set(ly, "elm.text", "No Item");

	it = elm_naviframe_item_push(ad->nf, _("No Items"), NULL, NULL, ly, NULL);
	elm_naviframe_item_title_enabled_set(it, EINA_FALSE, EINA_FALSE);
}

static void _nocontent_title_enabled_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *ly, *rect;
	struct appdata *ad = (struct appdata *)data;
	if (ad == NULL) return;

	ly = elm_layout_add(obj);
	elm_layout_theme_set(ly, "layout", "nocontents", "default");
	evas_object_size_hint_weight_set (ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);
	rect = evas_object_rectangle_add(evas_object_evas_get(obj));
	evas_object_show(rect);
	elm_object_part_content_set(ly, "elm.swallow.icon", rect);
	elm_object_part_text_set(ly, "elm.text", "No Item");

	elm_object_signal_emit(ly, "elm,state,title,enable", "elm");
	elm_naviframe_item_push(ad->nf, "Title Enabled", NULL, NULL, ly, NULL);
}

static void _list_click(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) elm_list_selected_item_get(obj);
	if(it == NULL) return;

	elm_list_item_selected_set(it, EINA_FALSE);
}

static Evas_Object* _create_list_winset(struct appdata* ad)
{
	Evas_Object *li;
	int idx = 0;

	if(ad == NULL) return NULL;

	li = elm_list_add(ad->nf);
	elm_list_mode_set(li, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(li, "selected", _list_click, NULL);

	while (menu_its[ idx ].name != NULL) {
		elm_list_item_append(
				li,
				menu_its[ idx ].name,
				NULL,
				NULL,
				menu_its[ idx ].func,
				ad);
		++idx;
	}
	elm_list_go(li);

	return li;
}

void layout_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct appdata *ad;
	Evas_Object *list;

	ad = (struct appdata *) data;
	if(ad == NULL) return;

	list = _create_list_winset(ad);
	elm_naviframe_item_push(ad->nf, _("Layout"), NULL, NULL, list, "empty");
}

