/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "main.h"

static char
*gl_text_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part EINA_UNUSED)
{
	static char *slider_itemlist[] = {
		"Slider Normal Style",
		"Slider with Bell Icon",
		"Slider with Alert Icon",
		"Slider with System Icon",
		NULL
	};

	int index = ((int)(long)data) / 2;
	return strdup(slider_itemlist[index]);
}

static Evas_Object
*gl_content_get(void *data, Evas_Object *obj, const char *part EINA_UNUSED)
{
	Evas_Object *slider, *img, *layout;
	int index = ((int)(long)data);
	layout = elm_layout_add(obj);
	elm_layout_file_set(layout, ELM_DEMO_EDJ, "slider_layout");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	if (index == 1) {
		slider = elm_slider_add(layout);
		elm_slider_indicator_show_set(slider, EINA_FALSE);
		evas_object_size_hint_weight_set(slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_slider_indicator_format_set(slider, "%1.0f");
		elm_slider_min_max_set(slider, 0, 9);
		elm_slider_value_set(slider, 1);
		elm_object_part_content_set(layout,"slider", slider);
	} else if (index == 3) {
		slider = elm_slider_add(layout);
		elm_slider_indicator_show_set(slider, EINA_FALSE);
		evas_object_size_hint_weight_set(slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_slider_indicator_format_set(slider, "%1.0f");
		elm_slider_min_max_set(slider, 0, 9);
		elm_slider_value_set(slider, 5);
		img = elm_image_add(slider);
		elm_image_file_set(img, ICON_DIR"/tw_number_controller_icon_bell.png", NULL);
		evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		elm_object_part_content_set(slider, "elm.swallow.icon", img);
		elm_object_part_content_set(layout,"slider", slider);
	} else if (index == 5) {
		slider = elm_slider_add(layout);
		elm_slider_indicator_show_set(slider, EINA_FALSE);
		evas_object_size_hint_weight_set(slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_slider_indicator_format_set(slider, "%1.0f");
		elm_slider_min_max_set(slider, 0, 9);
		elm_slider_value_set(slider, 5);
		img = elm_image_add(slider);
		elm_image_file_set(img, ICON_DIR"/tw_number_controller_icon_alert.png", NULL);
		evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		elm_object_part_content_set(slider, "elm.swallow.icon", img);
		elm_object_part_content_set(layout,"slider", slider);
	} else if (index == 7) {
		slider = elm_slider_add(layout);
		elm_slider_indicator_show_set(slider, EINA_FALSE);
		evas_object_size_hint_weight_set(slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
		elm_slider_indicator_format_set(slider, "%1.0f");
		elm_slider_min_max_set(slider, 0, 9);
		elm_slider_value_set(slider, 5);
		img = elm_image_add(slider);
		elm_image_file_set(img, ICON_DIR"/tw_number_controller_icon_system.png", NULL);
		evas_object_size_hint_aspect_set(img, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
		elm_object_part_content_set(slider, "elm.swallow.icon", img);
		elm_object_part_content_set(layout,"slider", slider);
	}
	return layout;
}

static void
gl_realized_cb(void *data, Evas_Object *obj, void *event_info)
{
	Eina_List *items = NULL;
	Elm_Object_Item *item = event_info;

	const Elm_Genlist_Item_Class *itc = elm_genlist_item_item_class_get(item);
	if (!strcmp(itc->item_style, "dialogue/1icon")) {
		/* unregister item itself */
		elm_object_item_access_unregister(item);

		/* convey highlight to its content */
		Evas_Object *content;
		content = elm_object_item_part_content_get(item, "elm.icon");
		items = eina_list_append(items, content);
		elm_object_item_access_order_set(item, items);
	}
}

static Evas_Object*
create_genlist(Evas_Object* parent)
{
	static Elm_Genlist_Item_Class itc, itc2;
	Evas_Object *genlist;
	int index;
	const int NUM_OF_ITEMS = 8;
	genlist = elm_genlist_add(parent);

	itc.item_style = "1text";
	itc.func.text_get = gl_text_get;
	itc.func.content_get = NULL;
	itc.func.state_get = NULL;
	itc.func.del = NULL;

	itc2.item_style = "1icon";
	itc2.func.text_get = NULL;
	itc2.func.content_get = gl_content_get;
	itc2.func.state_get = NULL;
	itc2.func.del = NULL;

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, 0.0);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, 0.0);
	elm_genlist_select_mode_set(genlist, ELM_OBJECT_SELECT_MODE_NONE);
	evas_object_show(genlist);

	for (index = 0; index < NUM_OF_ITEMS; index++) {
		if (index % 2 == 0) {
			elm_genlist_item_append(genlist, &itc, (void *) index, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		} else {
			elm_genlist_item_append(genlist, &itc2, (void *) index, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		}
	}

	evas_object_smart_callback_add(genlist, "realized", gl_realized_cb, NULL);
	return genlist;
}

void
slider_cb(void *data, Evas_Object *obj, void *event_info EINA_UNUSED)
{
	struct appdata *ad;

	ad = (struct appdata *) data;
	if (ad == NULL) return;
	Evas_Object *genlist;
	genlist = create_genlist(ad->nf);
	elm_naviframe_item_push(ad->nf, "Default Style", NULL, NULL, genlist, NULL);
}
